import { __decorate, __metadata } from 'tslib';
import { Component, ɵɵdefineInjectable, ɵɵinject, Injectable, NgModule, Injector, INJECTOR } from '@angular/core';
import { Validators, FormBuilder, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { ActivatedRoute, Router, RouterModule } from '@angular/router';
import { first, map, catchError, filter, take, switchMap, finalize, mergeMap, materialize, delay, dematerialize } from 'rxjs/operators';
import { AuthenticationService, NotificationType, NotificationService } from '@xdf/commons';
import { CommonModule } from '@angular/common';
import { TranslateModule, TranslateService } from '@ngx-translate/core';
import { throwError, of, BehaviorSubject } from 'rxjs';
import { HttpClient, HttpResponse } from '@angular/common/http';

let LoginComponent = class LoginComponent {
    constructor(formBuilder, route, router, authenticationService) {
        this.formBuilder = formBuilder;
        this.route = route;
        this.router = router;
        this.authenticationService = authenticationService;
        this.loading = false;
        this.submitted = false;
        this.error = '';
        // redirect to home if already logged
        if (this.authenticationService.currentUserValue) {
            this.router.navigate(['/home']);
        }
    }
    ngOnInit() {
        this.loginForm = this.formBuilder.group({
            username: ['', Validators.required],
            password: ['', Validators.required]
        });
        // get return url from route parameters or default to '/'
        const attReturnUrl = 'returnUrl';
        this.returnUrl = this.route.snapshot.queryParams[attReturnUrl] || '/';
    }
    get f() { return this.loginForm.controls; }
    onSubmit() {
        this.submitted = true;
        // stop here if form is invalid
        if (this.loginForm.invalid) {
            return;
        }
        this.loading = true;
        this.authenticationService.login(this.f.username.value, this.f.password.value)
            .pipe(first())
            .subscribe(data => {
            const returnUrl = JSON.parse(localStorage.getItem('returnUrl'));
            if (returnUrl) {
                this.returnUrl = returnUrl;
                localStorage.removeItem('returnUrl');
            }
            this.router.navigate([this.returnUrl]);
        }, error => {
            this.error = error;
            this.loading = false;
        });
    }
};
LoginComponent.ctorParameters = () => [
    { type: FormBuilder },
    { type: ActivatedRoute },
    { type: Router },
    { type: AuthenticationService }
];
LoginComponent = __decorate([
    Component({
        selector: 'xdf-example-login',
        template: "<div class=\"col-md-6 offset-md-3 mt-5\">\n    <div class=\"alert alert-info\">\n        Username: test<br />\n        Password: test\n    </div>\n    <div class=\"card\">\n        <h4 class=\"card-header\">{{ 'login.title' | translate }}</h4>\n        <div class=\"card-body\">\n            <form [formGroup]=\"loginForm\" (ngSubmit)=\"onSubmit()\">\n                <div class=\"form-group\">\n                    <label for=\"username\">{{ 'label.username' | translate }}</label>\n                    <input type=\"text\" formControlName=\"username\" class=\"form-control\" [ngClass]=\"{ 'is-invalid': submitted && f.username.errors }\" />\n                    <div *ngIf=\"submitted && f.username.errors\" class=\"invalid-feedback\">\n                        <div *ngIf=\"f.username.errors.required\">Username is required</div>\n                    </div>\n                </div>\n                <div class=\"form-group\">\n                    <label for=\"password\">{{ 'label.password' | translate }}</label>\n                    <input type=\"password\" formControlName=\"password\" class=\"form-control\" [ngClass]=\"{ 'is-invalid': submitted && f.password.errors }\" />\n                    <div *ngIf=\"submitted && f.password.errors\" class=\"invalid-feedback\">\n                        <div *ngIf=\"f.password.errors.required\">Password is required</div>\n                    </div>\n                </div>\n                <button [disabled]=\"loading\" class=\"btn btn-primary\">\n                    <span *ngIf=\"loading\" class=\"spinner-border spinner-border-sm mr-1\"></span>\n                    Login\n                </button>\n                <div *ngIf=\"error\" class=\"alert alert-danger mt-3 mb-0\">{{ error }}</div>\n            </form>\n        </div>\n    </div>\n  </div>\n  ",
        styles: [""]
    }),
    __metadata("design:paramtypes", [FormBuilder,
        ActivatedRoute,
        Router,
        AuthenticationService])
], LoginComponent);

class AuthorizationResource {
    constructor(program, control) {
        this.program = program;
        this.control = control;
    }
}
let AuthorizationService = class AuthorizationService {
    constructor(http) {
        this.http = http;
    }
    authorize(programId) {
        return this.http.get(`./service/security/controls/` + programId);
    }
};
AuthorizationService.ctorParameters = () => [
    { type: HttpClient }
];
AuthorizationService.ngInjectableDef = ɵɵdefineInjectable({ factory: function AuthorizationService_Factory() { return new AuthorizationService(ɵɵinject(HttpClient)); }, token: AuthorizationService, providedIn: "root" });
AuthorizationService = __decorate([
    Injectable({
        providedIn: 'root'
    }),
    __metadata("design:paramtypes", [HttpClient])
], AuthorizationService);

let AuthorizationResolver = class AuthorizationResolver {
    constructor(authorizationService, notificationService) {
        this.authorizationService = authorizationService;
        this.notificationService = notificationService;
    }
    resolve(route) {
        const key = 'program';
        let program = route.data[key];
        if (!program) {
            this.notificationService.showMessage(`Route requires \'program\' parameter`, 'Security Error!', NotificationType.error);
            return throwError(`Not Found \'program\' Parameter`);
        }
        return this.authorizationService.authorize(program);
    }
};
AuthorizationResolver.ctorParameters = () => [
    { type: AuthorizationService },
    { type: NotificationService }
];
AuthorizationResolver = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [AuthorizationService,
        NotificationService])
], AuthorizationResolver);

let XdfSecurityModule = class XdfSecurityModule {
};
XdfSecurityModule = __decorate([
    NgModule({
        declarations: [
            LoginComponent
        ],
        imports: [
            FormsModule,
            CommonModule,
            RouterModule,
            ReactiveFormsModule,
            TranslateModule
        ],
        exports: [
            LoginComponent
        ],
        providers: [
            AuthorizationResolver
        ]
    })
], XdfSecurityModule);

/**
 * @description
 * Permite validar que antes de usar el mantenimiento el usuario esté logueado en la aplicación.
 * La presente implementación supone que el login es parte de los routes configurados.
 */
let AuthGuard = class AuthGuard {
    constructor(router, authenticationService) {
        this.router = router;
        this.authenticationService = authenticationService;
    }
    canLoad() {
        const currentUser = this.authenticationService.currentUserValue;
        const response = !(!currentUser);
        return of(response);
    }
    canActivate(route, state) {
        const currentUser = this.authenticationService.currentUserValue;
        if (currentUser) {
            // logged in so return true
            return true;
        }
        // not logged in so redirect to login page with the return url
        if (state.url) {
            localStorage.setItem('returnUrl', JSON.stringify(state.url));
        }
        this.router.navigate(['login']);
        return false;
    }
};
AuthGuard.ctorParameters = () => [
    { type: Router },
    { type: AuthenticationService }
];
AuthGuard.ngInjectableDef = ɵɵdefineInjectable({ factory: function AuthGuard_Factory() { return new AuthGuard(ɵɵinject(Router), ɵɵinject(AuthenticationService)); }, token: AuthGuard, providedIn: "root" });
AuthGuard = __decorate([
    Injectable({
        providedIn: 'root'
    }),
    __metadata("design:paramtypes", [Router,
        AuthenticationService])
], AuthGuard);

let OAuthGuard = class OAuthGuard {
    constructor(router, authenticationService) {
        this.router = router;
        this.authenticationService = authenticationService;
    }
    canLoad() {
        return this.authenticationService.login(null, null).pipe(map(user => {
            if (!user) {
                window.location.href = './';
                return false;
            }
            return true;
        }));
    }
    canActivate() {
        return this.authenticationService.login(null, null).pipe(map(user => {
            if (!user) {
                window.location.href = './';
                return false;
            }
            return true;
        }));
    }
};
OAuthGuard.ctorParameters = () => [
    { type: Router },
    { type: AuthenticationService }
];
OAuthGuard.ngInjectableDef = ɵɵdefineInjectable({ factory: function OAuthGuard_Factory() { return new OAuthGuard(ɵɵinject(Router), ɵɵinject(AuthenticationService)); }, token: OAuthGuard, providedIn: "root" });
OAuthGuard = __decorate([
    Injectable({
        providedIn: 'root'
    }),
    __metadata("design:paramtypes", [Router,
        AuthenticationService])
], OAuthGuard);

let ResourceAuthGuard = class ResourceAuthGuard {
    constructor(authorizationService, notificationService, translate) {
        this.authorizationService = authorizationService;
        this.notificationService = notificationService;
        this.translate = translate;
    }
    canActivate(route, state) {
        const key = 'program';
        let program = route.data[key];
        if (!program) {
            this.notificationService.showMessage(this.translate.instant('security.message.error.parameter', { parameter: key }), this.translate.instant('security.message.error.title'), NotificationType.error);
            return throwError(this.translate.instant('security.message.error.parameter.not.found', { parameter: key }));
        }
        return this.authorizationService.authorize(program).pipe(map(data => {
            if (!data || data.length === 0) {
                this.notificationService.showMessage(this.translate.instant('message.error.resource.authorization'), this.translate.instant('title.error'), NotificationType.error);
                return false;
            }
            const keyAuthorization = 'authorization';
            route.paramMap[keyAuthorization] = data;
            return true;
        }));
    }
};
ResourceAuthGuard.ctorParameters = () => [
    { type: AuthorizationService },
    { type: NotificationService },
    { type: TranslateService }
];
ResourceAuthGuard.ngInjectableDef = ɵɵdefineInjectable({ factory: function ResourceAuthGuard_Factory() { return new ResourceAuthGuard(ɵɵinject(AuthorizationService), ɵɵinject(NotificationService), ɵɵinject(TranslateService)); }, token: ResourceAuthGuard, providedIn: "root" });
ResourceAuthGuard = __decorate([
    Injectable({
        providedIn: 'root'
    }),
    __metadata("design:paramtypes", [AuthorizationService,
        NotificationService,
        TranslateService])
], ResourceAuthGuard);

let ByteAuthenticationService = class ByteAuthenticationService extends AuthenticationService {
    constructor(http, injector) {
        super();
        this.http = http;
        this.injector = injector;
    }
    login(username, password) {
        return this.http.post(`http://localhost:4200/users/authenticate`, { username, password })
            .pipe(map(user => {
            // store user details and jwt token in local storage to keep user logged in between page refreshes
            localStorage.setItem('currentUser', JSON.stringify(user));
            if (this.currentUserSubject) {
                this.currentUserSubject.next(user);
            }
            else {
                this.currentUserSubject = new BehaviorSubject(user);
            }
            this.currentUser = this.currentUserSubject.asObservable();
            return user;
        }));
    }
    logout() {
        // remove user from local storage to log user out
        localStorage.removeItem('currentUser');
        localStorage.removeItem('menuInfo');
        this.currentUserSubject.next(null);
        const router = this.injector.get(Router);
        router.navigate(['/login']);
        return of();
    }
};
ByteAuthenticationService.ctorParameters = () => [
    { type: HttpClient },
    { type: Injector }
];
ByteAuthenticationService.ngInjectableDef = ɵɵdefineInjectable({ factory: function ByteAuthenticationService_Factory() { return new ByteAuthenticationService(ɵɵinject(HttpClient), ɵɵinject(INJECTOR)); }, token: ByteAuthenticationService, providedIn: "root" });
ByteAuthenticationService = __decorate([
    Injectable({
        providedIn: 'root'
    }),
    __metadata("design:paramtypes", [HttpClient, Injector])
], ByteAuthenticationService);

let OAuthAuthenticationService = class OAuthAuthenticationService extends AuthenticationService {
    constructor(http, injector) {
        super();
        this.http = http;
        this.injector = injector;
    }
    login(username, password) {
        return this.http.get('./service/oauth/userinfo', { withCredentials: true }).pipe(map((response) => {
            const key = 'userAuthentication';
            if (response && response[key]) {
                let user = response[key].details.principal;
                if (!user || user instanceof String) {
                    user = response[key].details;
                }
                if (!user) {
                    const router = this.injector.get(Router);
                    console.error('No se ha identificado el usuario registrado');
                    router.navigate(['/notfound']);
                    return null;
                }
                localStorage.setItem('currentUser', JSON.stringify(user));
                if (this.currentUserSubject) {
                    this.currentUserSubject.next(user);
                }
                else {
                    this.currentUserSubject = new BehaviorSubject(user);
                }
                this.currentUser = this.currentUserSubject.asObservable();
                return user;
            }
            else {
                return null;
            }
        }));
    }
    logout() {
        localStorage.removeItem('currentUser');
        localStorage.removeItem('menuInfo');
        this.currentUserSubject.next(null);
        window.location.href = './logout';
    }
};
OAuthAuthenticationService.ctorParameters = () => [
    { type: HttpClient },
    { type: Injector }
];
OAuthAuthenticationService.ngInjectableDef = ɵɵdefineInjectable({ factory: function OAuthAuthenticationService_Factory() { return new OAuthAuthenticationService(ɵɵinject(HttpClient), ɵɵinject(INJECTOR)); }, token: OAuthAuthenticationService, providedIn: "root" });
OAuthAuthenticationService = __decorate([
    Injectable({
        providedIn: 'root'
    }),
    __metadata("design:paramtypes", [HttpClient, Injector])
], OAuthAuthenticationService);

let AuthenticationInterceptor = class AuthenticationInterceptor {
    constructor() {
        this.AUTH_HEADER = 'Authorization';
        this.token = 'secrettoken';
        this.refreshTokenInProgress = false;
        this.refreshTokenSubject = new BehaviorSubject(null);
    }
    intercept(req, next) {
        if (!req.headers.has('Content-Type')) {
            req = req.clone({
                headers: req.headers.set('Content-Type', 'application/json')
            });
        }
        req = this.addAuthenticationToken(req);
        return next.handle(req).pipe(catchError(err => {
            if (err && err.status === 401) {
                if (this.refreshTokenInProgress) {
                    return this.refreshTokenSubject.pipe(filter(result => result !== null), take(1), switchMap(() => next.handle(this.addAuthenticationToken(req))));
                }
                else {
                    this.refreshTokenInProgress = true;
                    this.refreshTokenSubject.next(null);
                    return this.refreshAccessToken().pipe(switchMap((success) => {
                        this.refreshTokenSubject.next(success);
                        return next.handle(this.addAuthenticationToken(req));
                    }), finalize(() => this.refreshTokenInProgress = false));
                }
            }
            else {
                return throwError(err);
            }
        }));
    }
    refreshAccessToken() {
        return of('secret token');
    }
    addAuthenticationToken(request) {
        // If we do not have a token yet then we should not set the header.
        // Here we could first retrieve the token from where we store it.
        if (!this.token) {
            return request;
        }
        // If you are calling an outside domain then do not add the token.
        if (!request.url.match(/localhost:4200\//)) {
            return request;
        }
        return request.clone({
            headers: request.headers.set(this.AUTH_HEADER, 'Bearer ' + this.token)
        });
    }
};
AuthenticationInterceptor = __decorate([
    Injectable()
], AuthenticationInterceptor);

let AuthorizationInterceptor = class AuthorizationInterceptor {
    constructor() { }
    intercept(req, next) {
        return next.handle(req).pipe(catchError((error) => {
            return throwError(error);
        }));
    }
};
AuthorizationInterceptor = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], AuthorizationInterceptor);

const users = [{ username: 'test', password: 'test', firstName: 'Test', lastName: 'User' }];
let AuthenticationFakeBackendInterceptor = class AuthenticationFakeBackendInterceptor {
    intercept(request, next) {
        const { url, method, headers, body } = request;
        return of(null)
            .pipe(mergeMap(handleRoute))
            .pipe(materialize())
            .pipe(delay(500))
            .pipe(dematerialize());
        function handleRoute() {
            switch (true) {
                case url.endsWith('/users/authenticate') && method === 'POST':
                    return authenticate();
                case url.endsWith('/users') && method === 'GET':
                    return getUsers();
                default:
                    // pass through any requests not handled above
                    return next.handle(request);
            }
        }
        function authenticate() {
            const { username, password } = body;
            const user = users.find(x => x.username === username && x.password === password);
            if (!user) {
                return error('Username or password is incorrect');
            }
            return ok({
                username: user.username,
                firstName: user.firstName,
                lastName: user.lastName,
                token: 'fake-jwt-token'
            });
        }
        function getUsers() {
            if (!isLoggedIn()) {
                return unauthorized();
            }
            return ok(users);
        }
        // helper functions
        function ok(bodyContent) {
            return of(new HttpResponse({ status: 200, body: bodyContent }));
        }
        function error(message) {
            return throwError({ error: { message } });
        }
        function unauthorized() {
            return throwError({ status: 401, error: { message: 'Unauthorised' } });
        }
        function isLoggedIn() {
            return headers.get('Authorization') === 'Bearer fake-jwt-token';
        }
    }
};
AuthenticationFakeBackendInterceptor = __decorate([
    Injectable()
], AuthenticationFakeBackendInterceptor);

const basePath = './service/authorize';
const data = ['new', 'view', 'delete', 'edit', 'export'];
let AuthorizationFakeBackendInterceptor = class AuthorizationFakeBackendInterceptor {
    intercept(request, next) {
        const { url, method, headers, body } = request;
        return of(null)
            .pipe(mergeMap(handleRoute))
            .pipe(materialize())
            .pipe(dematerialize());
        function handleRoute() {
            switch (true) {
                case url.endsWith(basePath) && method === 'POST':
                    return authorize();
                default:
                    // pass through any requests not handled above
                    return next.handle(request);
            }
        }
        function authorize() {
            return ok(data);
        }
        // helper functions
        function ok(bodyContent) {
            return of(new HttpResponse({ status: 200, body: bodyContent }));
        }
        function error(message) {
            return throwError({ error: { message } });
        }
    }
};
AuthorizationFakeBackendInterceptor = __decorate([
    Injectable()
], AuthorizationFakeBackendInterceptor);

var programsData = [
	"CUSTOMER",
	"COUNTRY",
	"SYSTEM_SETTINGS",
	"USER_ROLE",
	"USER",
	"system_settings_information",
	"system_settings_password",
	"system_settings_userpolicy",
	"VALPOS",
	"APPLICATION",
	"HOBBY",
	"TAGS",
	"COMMISSSIONS"
];

const basePath$1 = '/service/security/programs';
let ProgramsFakeBackendInterceptor = class ProgramsFakeBackendInterceptor {
    intercept(request, next) {
        const { url, method, headers, body } = request;
        const data = programsData;
        return of(null)
            .pipe(mergeMap(handleRoute))
            .pipe(materialize())
            .pipe(delay(50))
            .pipe(dematerialize());
        function handleRoute() {
            switch (true) {
                case url.match('.*' + basePath$1) && method === 'GET':
                    return getList();
                default:
                    // pass through any requests not handled above
                    return next.handle(request);
            }
        }
        function getList() {
            return ok(data);
        }
        // helper functions
        function ok(bodyContent) {
            return of(new HttpResponse({ status: 200, body: bodyContent }));
        }
        function error(message) {
            return throwError({ error: { message } });
        }
    }
};
ProgramsFakeBackendInterceptor = __decorate([
    Injectable()
], ProgramsFakeBackendInterceptor);

var controlsData = [
	"new",
	"view",
	"edit",
	"delete",
	"export",
	"audit"
];

const basePath$2 = '/service/security/controls';
let ControlsByProgramsFakeBackendInterceptor = class ControlsByProgramsFakeBackendInterceptor {
    intercept(request, next) {
        const { url, method, headers, body } = request;
        const data = controlsData;
        return of(null)
            .pipe(mergeMap(handleRoute))
            .pipe(materialize())
            .pipe(delay(50))
            .pipe(dematerialize());
        function handleRoute() {
            switch (true) {
                case url.indexOf(basePath$2) > -1 && method === 'GET':
                    return getList();
                default:
                    // pass through any requests not handled above
                    return next.handle(request);
            }
        }
        function getList() {
            return ok(data);
        }
        // helper functions
        function ok(bodyContent) {
            return of(new HttpResponse({ status: 200, body: bodyContent }));
        }
        function error(message) {
            return throwError({ error: { message } });
        }
    }
};
ControlsByProgramsFakeBackendInterceptor = __decorate([
    Injectable()
], ControlsByProgramsFakeBackendInterceptor);

/*
 * Public API Surface of xdf-security
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AuthGuard, AuthenticationFakeBackendInterceptor, AuthenticationInterceptor, AuthorizationFakeBackendInterceptor, AuthorizationInterceptor, AuthorizationResolver, AuthorizationResource, AuthorizationService, ByteAuthenticationService, ControlsByProgramsFakeBackendInterceptor, LoginComponent, OAuthAuthenticationService, OAuthGuard, ProgramsFakeBackendInterceptor, ResourceAuthGuard, XdfSecurityModule };
//# sourceMappingURL=xdf-security.js.map
